<?php
/*
 * Plugin Name: Custom GlotPress Source
 * Plugin URI: https://git.avecnous.eu/wordpress/custom-glotpress-source
 * Description: Allows to manage translations from a custom GlotPress install.
 * Version: 1.1
 * Author: N.O.U.S. Open Useful and Simple
 * Author URI: https://avecnous.eu/
 * Text Domain: custom-glotpress-source
 * Domain Path: wp/custom-glotpress-source
 */


global $Custom_GlotPress_Source;

class Custom_GlotPress_Source{
    var $options;
    var $defaults;
    var $last_check_date;
    var $available_translations = array();
    var $mu = false;

    function __construct(){
        require_once(ABSPATH.'/wp-admin/includes/plugin.php');
        $this->mu = is_plugin_active_for_network(basename(__DIR__).'/'.basename(__FILE__));
        add_action(($this->mu ? 'network_' : '').'admin_menu', array(&$this, 'admin_menu'));
        add_action('admin_init', array(&$this, 'admin_init'));
        add_action('admin_post_update', array($this, 'trigger_network_settings') );

        add_action('core_upgrade_preamble', array(&$this, 'core_upgrade_preamble'));
        add_action('update-core-custom_do-custom-translation-upgrade', array(&$this, 'update_core'));

        $this->defaults = array(
            'url'=>'',
        );

        $this->last_check_date = $this->get_option('custom-glotpress-last-check');
    }

    function get_option($option){
        $func = ($this->mu ? 'get_site_option' : 'get_option');
        return $func($option);
    }

    function update_option($option, $value){
        $func = ($this->mu ? 'update_site_option' : 'update_option');
        return $func($option, $value);
    }

    function Options(){
        if(!$this->options){
            $this->options = wp_parse_args((array) $this->get_option('custom-glotpress-src'), $this->defaults);
        }
        return $this->options;
    }

    function Get($setting){
        return (isset($this->Options()[$setting]) ? $this->Options()[$setting] : false);
    }

    function admin_menu() {
        add_submenu_page($this->mu ? 'settings.php' : 'options-general.php',__('Custom GlotPress Source', 'custom-glotpress-source'), __('GlotPress', 'custom-glotpress-source'), 'manage_options', 'ctm-gp-src-settings', array(&$this, 'manage_settings'));
    }

    function admin_init() {
        register_setting('ctm-gp-src-settings', 'custom-glotpress-src');

        add_settings_section(
                'ctm-gp-src-settings-basics', __('Basics', 'custom-glotpress-source'), array(&$this, 'settings_section_callback'), 'ctm-gp-src-settings'
        );

        add_settings_field(
                'custom-glotpress-src_url',
                __('API URL', 'custom-glotpress-source'),
                array(&$this, 'settings_field_default_callback'),
                'ctm-gp-src-settings',
                'ctm-gp-src-settings-basics',
                array(
                    'name' => 'url',
                    'description' => __('example: https://mysite.com/glotpress/', 'custom-glotpress-source')
                    )
        );
    }

    public function trigger_network_settings(){
        if(!$this->mu){
            return;
        }
        if(!isset($_POST['custom-glotpress-src'])){
            return;
        }
        if (!wp_verify_nonce(\filter_input(INPUT_POST, 'ctm-gp-src-settings', FILTER_SANITIZE_STRING), 'ctm-gp-src-settings')) {
            wp_die(__('Cheating uh?', 'custom-glotpress-source'));
        }
        $settings = array(
            'url'=>esc_url_raw($_POST['custom-glotpress-src']['url']),
        );
        $function = $this->mu ? 'update_site_option' : 'update_option';
        $update = $function( 'custom-glotpress-src', $settings);
        wp_redirect(
            add_query_arg(
                array(
                    'page' => 'ctm-gp-src-settings',
                    'confirm' => $update,
                ),
                (admin_url( $this->mu ? 'network/settings.php' : 'options-general.php' ))
            )
        );
        exit;
    }

    function manage_settings() {
        ?>
        <div class="wrap">
            <h2><?php _e('Custom GlotPress Settings', 'custom-glotpress-source'); ?></h2>
            <form action="<?php echo admin_url($this->mu ? 'admin-post.php' : 'options.php'); ?>" method="post">
                <?php wp_nonce_field('ctm-gp-src-settings', 'ctm-gp-src-settings'); ?>
                <?php settings_fields('ctm-gp-src-settings'); ?>
                <?php do_settings_sections('ctm-gp-src-settings'); ?>
                <?php submit_button(); ?>
            </form>
        </div>
        <?php
    }

    function settings_section_callback($arg) {

    }

    function settings_field_default_callback($args) {
        $options = $this->Options();
        $option = 'custom-glotpress-src';
        ?>
        <input name="<?php echo $option; ?>[<?php esc_attr_e($args['name']); ?>]" id="<?php esc_attr_e($args['name']); ?>" value="<?php esc_attr_e( isset($options[$args['name']]) ? $options[$args['name']] : ''); ?>" class="regular-text"/>
        <?php if (isset($args['description']) && $args['description']): ?>
            <div class="description"><?php esc_html_e($args['description']); ?></div>
        <?php endif; ?>
        <?php
    }


    function core_upgrade_preamble(){
        if(false != $count_success = filter_input(INPUT_GET, 'upgrade-custom-t9n-success', FILTER_SANITIZE_NUMBER_INT)){
            show_message(sprintf(__('Upgrade successful for %d translations', 'custom-glotpress-source'), $count_success));
        }
        if(false != $count_fail = filter_input(INPUT_GET, 'upgrade-custom-t9n-fail', FILTER_SANITIZE_NUMBER_INT)){
            show_message(sprintf(__('Upgrade failed for %d translations', 'custom-glotpress-source'), $count_fail));
        }

        $this->check_translations();
        if(!count($this->available_translations)){
            return;
        }
        ?>

        <form method="post" action="update-core.php?action=do-custom-translation-upgrade" name="custom-upgrade-translations" class="upgrade">
            <p><?php _e('New custom translations are available', 'custom-glotpress-source'); ?></p>
            <?php wp_nonce_field('ctm-gp-src-upgrade', 'ctm-gp-src-upgrade'); ?>
            <p><input class="button" type="submit" value="<?php esc_attr_e(__('Upgrade translations', 'custom-glotpress-source')); ?>" name="upgrade"></p>
        </form>
        <?php
    }

    function update_core(){
        if (!wp_verify_nonce(\filter_input(INPUT_POST, 'ctm-gp-src-upgrade', FILTER_SANITIZE_STRING), 'ctm-gp-src-upgrade')) {
            wp_die(__('Cheating uh?', 'custom-glotpress-source'));
        }
        $projects = $this->check_projects();
        foreach($projects as $remote_path=>$local_path){
            $this->check_translations($remote_path);
        }
        $count_success=0;
        $count_fail=0;
        foreach($this->available_translations as $set){
            $mo_file = $this->download($set->path, $set->locale, $set->slug, 'mo');
            $final_mo_path = WP_LANG_DIR.'/'.$projects[$set->path].'-'.$set->wp_locale.'.mo';
            if(file_put_contents($final_mo_path, $mo_file)){
                $count_success++;
            }
            else{
                $count_fail++;
            }
        }
        $this->update_option('custom-glotpress-last-check', time());
        wp_safe_redirect(add_query_arg(array('upgrade-custom-t9n-success'=>$count_success, 'upgrade-custom-t9n-fail'=>$count_fail), admin_url(($this->mu ? 'network/' : '').'update-core.php')));
        exit;
    }

    function check_projects(){
        $projects = array();
        $plugins = get_plugins();
        foreach ($plugins as $plugin) {
            if(isset($plugin['DomainPath']) && !empty($plugin['DomainPath']) && $plugin['DomainPath']!='/languages' && $plugin['DomainPath']!='/languages/'){
                $projects[$plugin['DomainPath']] = 'plugins/'.$plugin['TextDomain'];
            }
        }
        $themes = get_themes();
        foreach ($themes as $theme) {
            $domain_path = $theme->get('DomainPath');
            if(!$domain_path){
                continue;
            }
            if(!empty($domain_path) && $domain_path!='/languages' && $domain_path!='/languages/'){
                $projects[$domain_path] = 'themes/'.$theme->get('TextDomain');
            }
        }
        return $projects;
    }

    function check_translations($project_name=''){
        $base_url = $this->Get('url');
        $project_url = "{$base_url}api/projects/{$project_name}";

        $request = wp_remote_get( $project_url );

        if( is_wp_error( $request ) ) {
            return;
        }
        $body = wp_remote_retrieve_body( $request );
        $data = json_decode( $body );

        if(isset($data->translation_sets) && is_array($data->translation_sets)){
            foreach($data->translation_sets as $translation_set){
                if(strtotime($translation_set->last_modified) > $this->last_check_date){
                    $translation_set->path = $project_name;
                    $this->available_translations[$translation_set->id] = $translation_set;
                }
            }
        }

        $sub_projects = (empty($project_name) ? $data : $data->sub_projects);
        if(is_array($sub_projects)){
            foreach($sub_projects as $sub_project){
                $this->check_translations($sub_project->path);
            }
        }
    }

    function download($project_path, $locale, $set='default', $format='mo'){
        $base_url = $this->Get('url');
        $file_url = "{$base_url}/api/projects/{$project_path}/{$locale}/{$set}/export-translations?format={$format}";
        $request = wp_remote_get( $file_url );

        if( is_wp_error( $request ) ) {
            return;
        }
        return wp_remote_retrieve_body( $request );
    }
}

$Custom_GlotPress_Source = new Custom_GlotPress_Source;
